local edit = require 'edit'
require 'widgets'

local Margin_top = 15
local Margin_left = 25
local Margin_right = 25

local Editor  -- most data will be here

local Screen_width, Screen_height

function love.load(arg)
  if not edit.is_this_love_version_supported() then
    local major_version, minor_version = love.getVersion()
    error(
      ("This app hasn't been tested with LÖVE version %d.%d. Please add %d.%d to `is_this_love_version_supported` in edit.lua to try it out, and switch to version %s if you run into issues."):format(major_version, minor_version, major_version, minor_version, edit.preferred_love_version()))
  end

  love.keyboard.setKeyRepeat(true)

  love.graphics.setBackgroundColor(1,1,1)

  edit.load()

  -- editor remembers settings across restarts
  local settings = edit.load_settings()
  if settings == nil then
    Screen_width, Screen_height = love.window.getMode()
    local right_margin = assign_space_for_tabs()
    Editor = edit.new_from_defaults(Margin_top, Margin_left, Screen_width-right_margin-Margin_right, Screen_height-Margin_top)
    if #arg > 0 then
      edit.load_file(Editor, arg[1])
    end
  else
    Screen_width, Screen_height = settings.width, settings.height
    local right_margin = assign_space_for_tabs()
    Editor = edit.new_from_settings(settings, Margin_top, Margin_left, Screen_width-right_margin-Margin_right, settings.height-Margin_top)
    if #arg > 0 and Editor.filename ~= edit.absolutize(arg[1]) then
      edit.load_file(Editor, arg[1])
    end
  end

  refresh_tab_widgets()

  if #arg > 1 then
    print('ignoring commandline args after '..arg[1])
  end

  love.window.setTitle('text2.love - '..Editor.filename)
end

function love.filedropped(file)
  -- first make sure to autosave edits on any existing file
  edit.final_save(Editor)

  -- clear the slate for the new file
  edit.load_file(Editor, file:getFilename())

  love.window.setTitle('text2.love - '..Editor.filename)
end

function love.resize(w,h)
  Screen_width, Screen_height = w, h
  local right_margin = assign_space_for_tabs()
  edit.resize(Editor, w, h, w-right_margin-Margin_right, h-Margin_top)
end

function love.draw()
  love.graphics.setFont(Editor.font)
  edit.draw(Editor)
  draw_tabs(Editor)
end

function assign_space_for_tabs()
  return math.max(Screen_width*0.3, 100)
end

function refresh_tabs()
  local tabs = {}
  for i,line in ipairs(Editor.lines) do
    if line:match('^%-%-%-') then
      local tab, _ = line:gsub('^%-%-%-%s*', '')
      table.insert(tabs, {name=tab, line=i})
    end
  end
  return tabs
end

function draw_tabs(Editor)
  local right_margin = assign_space_for_tabs()
  love.graphics.setColor(0.8,0.8,0.8)
  local x = Screen_width-right_margin
  love.graphics.line(x, 0, x, Screen_height)
  widgets.__draw()
end

function refresh_tab_widgets()
  clear_widgets()
  local right_margin = assign_space_for_tabs()
  local tabs = refresh_tabs(Editor)
  local x = Screen_width-right_margin
  local y = Margin_top
  for _,tab in ipairs(tabs) do
    if widgets[tab.name] == nil then
      widgets[tab.name] = tab_button(tab.name, tab.line, x, y)
      y = y + Editor.line_height+10
    end
  end
end

function tab_button(name, line_index, x, y)
  local w = Editor.font:getWidth(name)+10
  local draw = function()
    love.graphics.setColor(0.8,0.8,0.8)
    love.graphics.rectangle('line', x,y, w, Editor.line_height)
    love.graphics.print(name, x+5, y)
  end
  local ispress = function(x2,y2)
    return x2 >= x and x2 <= x+w and y2 >= y and y2 <= y+Editor.line_height
  end
  local press = function()
    Editor.screen_top = {line=line_index, pos=1}
    Editor.cursor = {line=line_index, pos=1}
  end
  return {draw=draw, ispress=ispress, press=press}
end

function clear_widgets()
  if widgets then
    for k,v in pairs(widgets) do
      if k:find('__') ~= 1 then
        widgets[k] = nil
      end
    end
  end
end

function love.mousepressed(x,y, mouse_button)
  love.keyboard.setTextInput(true)  -- bring up keyboard on touch screen
  if x < Editor.right then
    edit.on_mouse_pressed(Editor, x,y, mouse_button)
  end
  widgets.__mousepressed(x,y, b)
end

function love.mousereleased(x,y, mouse_button)
  if x < Editor.right then
    edit.on_mouse_released(Editor, x,y, mouse_button)
  end
  widgets.__mousereleased(x,y, b)
end

function love.wheelmoved(dx,dy)
  edit.on_mouse_wheel_moved(Editor, dx,dy)
end

function love.keypressed(key, scancode, isrepeat)
  edit.on_key_pressed(Editor, key, scancode, isrepeat)
  refresh_tab_widgets()
end

function love.textinput(t)
  edit.on_text_input(Editor, t)
  refresh_tab_widgets()
end

function love.keyreleased(key, scancode)
  edit.on_key_released(Editor, key, scancode)
end

function love.update(dt)
  edit.maybe_autosave(Editor, dt)
  widgets.__update(dt)
end

function love.quit()
  edit.save_settings(Editor)
  edit.final_save(Editor)
end

function love.focus(in_focus)
  edit.on_focus(in_focus)
end
